import { Package } from './package';
import { Geolocation } from '@capacitor/geolocation';
import { createSerialOperation } from '@deep-foundation/deeplinks/imports/gql/index.js';
import debug from 'debug';
/**
 *
 * @example
 * #### Create a decorator from another decorator
```ts
const deep = new DeepClient({token: ''});
const anotherDeepDecorator = createAnotherDecorator(deep); // Note that this step is optional and showed only to demonstrate that you can create a decorator from another decorator and have a chain of decorators
const geolocationDeepDecorator = createAdditionalFeatureDecorator(anotherDeepDecorator);
```
 */
export function createGeolocationDecorator(deep) {
    const log = debug(`@deep-foundation/capacitor-geolocation:${createGeolocationDecorator.name}`);
    log({ deep });
    const _package = new Package({ deep });
    log({ _package });
    return Object.assign({
        ...deep,
        "@deep-foundation/capacitor-geolocation": _package,
        capacitorGeolocationPackage: _package,
        requiredPackagesInMinilinksToApply: [
            ...('requiredPackagesInMinilinksToApply' in deep ? deep.requiredPackagesInMinilinksToApply : []),
            _package.name,
            "@deep-foundation/core"
        ],
        async applyRequiredPackagesInMinilinks() {
            const log = debug(`@deep-foundation/capacitor-geolocation:GeolocationDecorator:${this.applyRequiredPackagesInMinilinks.name}`);
            const { data: links } = await deep.select({
                up: {
                    tree_id: {
                        _id: ["@deep-foundation/core", "containTree"]
                    },
                    parent: {
                        _or: this.requiredPackagesInMinilinksToApply.map((packageName) => ({
                            id: {
                                _id: [packageName]
                            }
                        }))
                    }
                }
            });
            log({ links });
            const minilinksApplyResult = deep.minilinks.apply(links);
            log({ minilinksApplyResult });
            this.requiredPackagesInMinilinksToApply = [];
            return minilinksApplyResult;
        },
        async insertPosition(options) {
            const log = debug(`@deep-foundation/capacitor-geolocation:GeolocationDecorator:${this.insertPosition.name}`);
            log({ options });
            const { position, containerLinkId, id } = options;
            const insertOperations = await this.makePositionInsertOperations({ position: position, containerLinkId: containerLinkId, id: id });
            log({ insertOperations });
            const serialResult = await this.serial({ operations: insertOperations });
            log({ serialResult });
            return serialResult;
        },
        async makePositionInsertOperations(options) {
            const log = debug(`@deep-foundation/capacitor-geolocation:GeolocationDecorator:${this.makePositionInsertOperations.name}`);
            log({ options });
            const { id } = options;
            const operation = createSerialOperation({
                ...(id ? { id } : {}),
                type: 'insert',
                table: 'links',
                objects: {
                    type_id: _package.Position.idLocal(),
                    object: {
                        data: {
                            value: options.position
                        }
                    },
                    in: {
                        data: {
                            type_id: deep.idLocal("@deep-foundation/core", "Contain"),
                            from_id: options.containerLinkId ?? deep.linkId,
                        }
                    }
                },
            });
            log({ operation });
            return [operation];
        },
        async updatePosition(options) {
            const operations = await this.makeUpdatePositionOperations(options);
            return await deep.serial({
                operations
            });
        },
        async makeUpdatePositionOperations(options) {
            const log = debug(`@deep-foundation/capacitor-geolocation:GeolocationDecorator:${this.makeUpdatePositionOperations.name}`);
            log({ options });
            const operation = createSerialOperation({
                type: 'update',
                table: 'objects',
                value: {
                    value: options.position
                },
                exp: {
                    link_id: options.id
                }
            });
            log({ operation });
            return [operation];
        },
        async getPosition(options) {
            const log = debug(`@deep-foundation/capacitor-geolocation:GeolocationDecorator:${this.getPosition.name}`);
            const { data: [positionLink] } = await deep.select(options.linkId);
            if (!positionLink) {
                throw new Error(`Failed to find link with id ##${options.linkId}`);
            }
            return positionLink.value?.value;
        },
        async watchPosition(options) {
            const { containerLinkId, watchPositionOptions } = options;
            return await Geolocation.watchPosition(watchPositionOptions, async (position, error) => {
                if (error) {
                    throw error;
                }
                await this.insertPosition({ position, containerLinkId });
            });
        },
        async clearWatch(options) {
            await Geolocation.clearWatch(options);
        },
        async checkPermissions() {
            const log = debug(`@deep-foundation/capacitor-geolocation:${this.checkPermissions.name}`);
            const permissionsStatus = await Geolocation.checkPermissions();
            log({ permissionsStatus });
            return permissionsStatus;
        },
        async requestPermissions() {
            const log = debug(`@deep-foundation/capacitor-geolocation:${this.requestPermissions.name}`);
            const permissionsStatus = await Geolocation.requestPermissions();
            log({ permissionsStatus });
            return permissionsStatus;
        }
    }, deep);
}
